%% Master Thesis - Non-linear Wave Impact on Monopile Structures

%  Philipp Schpfer - KTH (Stockholm) & NTNU (Trondheim)

%  In Corporation With Ramboll Denmark AS, Esbjerg

%  15.06.2016 -  Validation
%  Matlab vs. WAVGEN & WAVGEN vs. OCW3D
 
clear all
close all
clc

%% Reading data
% M = Matlab; W = WAVGEN; O = OCW3D
% sin = sinusoidal; st = Stokes' 5th; sf = stream function 
% d = deep water; s = shallow water

data = xlsread('Validation.xls');

%% Surface elevation: Matlab vs. WAVGEN

x1 = data(1:201,1);              % [m] - x-position

M_sin = data(1:201,2);              % [m] - Sinusoidal wave by Matlab
W_sin = data(1:201,3);              % [m] - Sinusoidal wave by WAVGEN

M_st = data(1:201,4);              % [m] - Stokes' 5th wave by Matlab
W_st = data(1:201,5);              % [m] - Sinusoidal wave by WAVGEN

% Sinusoidal wave
figure(1)
plot(x1,M_sin,'-b','LineWidth',4);
hold on
plot(x1,W_sin,'--g','LineWidth',2.5);
graph2d.constantline(0);
grid on
axis([-100 100 -4 4]);
xlabel('x-position, x [m]');
ylabel('Surface elevation, \eta [m]');
legend('Matlab','WAVGEN','Location','SouthEast');
title('Linear wave - Matlab vs. WAVGEN');
text(92,0.15,'SWL')

% Stokes' 5th order wave
figure(2)
plot(x1,M_st,'-b','LineWidth',4);
hold on
plot(x1,W_st,'--g','LineWidth',2.5);
graph2d.constantline(0);
grid on
axis([-100 100 -4 4]);
xlabel('x-position, x [m]');
ylabel('Surface elevation, \eta [m]');
legend('Matlab','WAVGEN','Location','SouthEast');
title('Stokes 5^t^h order wave - Matlab vs. WAVGEN');
text(92,0.15,'SWL')

%% Surface elevation: OCW3D vs. WAVGEN

t1 = data(1:301,6);              % [s] - Time
t11 = 0:0.1:30;
O_sf_d = data(1:301,7);              % [m] - Stream function wave (deep) by OCW3D

O_sf_s = data(1:301,8);              % [m] - Stream function wave (shallow) by OCW3D

W_sf_d = data(1:301,9);              % [m] - Stream function wave (deep) by WAVGEN

W_sf_s = data(1:301,10);              % [m] - Stream function wave (shallow) by WAVGEN

% Deep water
figure(3)
plot(t1,O_sf_d,'-b','LineWidth',4);
hold on
plot(t1,W_sf_d,'--g','LineWidth',2.5);
graph2d.constantline(0);
grid on
axis([0 200 -4 4]);
xlabel('x-position, x [m]');
ylabel('Surface elevation, \eta [m]');
legend('OCW3D','WAVGEN','Location','SouthEast');
title('Stream function wave in deep water - OCW3D vs. WAVGEN');
text(193,0.15,'SWL')

% Shallow water
figure(4)
plot(t1-50,O_sf_s,'-b','LineWidth',4);
hold on
plot(t1-50,W_sf_s,'--g','LineWidth',2.5);
graph2d.constantline(0);
grid on
axis([0 200 -4 4]);
xlabel('x-position, x [m]');
ylabel('Surface elevation, \eta [m]');
legend('OCW3D','WAVGEN','Location','SouthEast');
title('Stream function wave in shallow water - OCW3D vs. WAVGEN');
text(193,0.15,'SWL')

%% OTM: Matlab vs. WAVGEN

x2 = data(1:201,11);              % [m] - x-position

M_sin_O = data(1:201,12);              % [m] - OTM by sinusoidal wave by Matlab
W_sin_O = data(1:201,13);              % [m] - OTM by sinusoidal wave by WAVGEN

M_st_O = data(1:201,14);              % [m] - OTM by Stokes' 5th wave by Matlab
W_st_O = data(1:201,15);              % [m] - OTM by Sinusoidal wave by WAVGEN

% Sinusoidal wave
figure(5)
plot(x2,M_sin_O*-1,'-r','LineWidth',4);
hold on
plot(x2,W_sin_O*-1,'--y','LineWidth',2.5);
graph2d.constantline(0);
grid on
axis([-100 100 -8000 8000]);
xlabel('x-position, x [m]');
ylabel('Overturning moment, OTM [kNm]');
legend('Matlab','ROSA + WAVGEN','Location','SouthEast');
title('OTM by linear wave - Matlab vs. WAVGEN');
text(93,250,'SWL')

% Stokes' 5th order wave
figure(6)
plot(x2,M_st_O*-1,'-r','LineWidth',4);
hold on
plot(x2,W_st_O*-1,'--y','LineWidth',2.5);
graph2d.constantline(0);
grid on
axis([-100 100 -8000 8000]);
xlabel('x-position, x [m]');
ylabel('Overturning moment, OTM [kNm]');
legend('Matlab','ROSA + WAVGEN','Location','SouthEast');
title('OTM by Stokes 5^t^h order wave - Matlab vs. WAVGEN');
text(90,250,'SWL')

%% OTM: OCW3D vs. WAVGEN

t_d = data(1:436,16);              % [s] - time for deep waters

W_sf_d_O = data(1:436,17);              % [m] - OTM by stream function wave (deep) by WAVGEN

O_sf_d_O = data(1:436,18);              % [m] - OTM by stream function wave (deep) by OCW3D

eta_d = data(1:436,19)/1000;              % [m] - Surface elevation of irregular combined sea state

t_s = data(1:432,20);              % [s] - time for shallow waters

W_sf_s_O = data(1:432,21);              % [m] - OTM by stream function wave (shallow) by WAVGEN

O_sf_s_O = data(1:432,22);              % [m] - OTM by stream function wave (shallow) by OCW3D

eta_s = data(1:432,24);              % [m] - Surface elevation of irregular combined sea state

% Deep water
figure(7)
[AX,H1,H2] = plotyy(t_d,O_sf_d_O,t_d,eta_d);
grid on
hold on; 
plot(t_d,W_sf_d_O,'--y','LineWidth',3);
set(H1,'Color','r');
set(H2,'Color','b');
set(H1,'LineWidth',3);
set(H2,'LineWidth',2);
legend('ROSA + OCW3D','ROSA + WAVGEN','Wave profile','Location','SouthEast');

set(AX(1),'XLim', [30 65]);
set(AX(2),'XLim', [30 65]);
set(AX(1),'YLim', [-8000 8000]);
set(AX(1),'YTick', -8000:2000:8000);
set(AX(2),'YLim', [-4 4]);
set(AX(2),'YTick', -4:1:4);
set(AX(1),'YColor','k');
set(AX(2),'YColor','k');

set(get(AX(1),'Ylabel'),'string','Overturning moment, OTM [kNm]','Color','r');
set(get(AX(2),'Ylabel'),'string','Surface elevation, \eta [m]','Color','b');
xlabel('Time, s [m]');
title('OTM by stream function wave in deep water - OCW3D vs. WAVGEN');

% Shallow water
figure(8)
[AX,H1,H2] = plotyy(t_s,O_sf_s_O,t_s,eta_s);
grid on
hold on; 
plot(t_s,W_sf_s_O,'--y','LineWidth',3);
set(H1,'Color','r');
set(H2,'Color','b');
set(H1,'LineWidth',3);
set(H2,'LineWidth',2);
legend('ROSA + OCW3D','ROSA + WAVGEN','Wave profile','Location','SouthEast');

set(AX(1),'XLim', [30 70]);
set(AX(2),'XLim', [30 70]);
set(AX(1),'YLim', [-8000 8000]);
set(AX(1),'YTick', -8000:2000:8000);
set(AX(2),'YLim', [-4 4]);
set(AX(2),'YTick', -4:1:4);
set(AX(1),'YColor','k');
set(AX(2),'YColor','k');

set(get(AX(1),'Ylabel'),'string','Overturning moment, OTM [kNm]','Color','r');
set(get(AX(2),'Ylabel'),'string','Surface elevation, \eta [m]','Color','b');
xlabel('Time, s [m]');
title('OTM by stream function wave in shallow water - OCW3D vs. WAVGEN');

% End of Code