%% Master Thesis - Non-linear Wave Impact on Monopile Structures

%  Philipp Schpfer - KTH (Stockholm) & NTNU (Trondheim)

%  In Corporation With Ramboll Denmark AS, Esbjerg

%  17.02.2016 -  LINEAR WAVE THEORY
%  Regular wave - finite water depth
 
clear all
close all
clc

%% Input data

g = 9.81;                           % [m/s^2] - acceleration of gravity
rho = 1025.0;                       % [kg/m^3] - salt water density
N = 100;                            % [-] - number of regular wave components
numb_exp = 4;                       % [-] - number of exmaple wave components plotted

h = 10.0;                           % [m] - average water depth
T_z = 10;                           % [s] - mean zero-crossing wave period
H_s = 5.0;                          % [m] - significant wave height

gamma = 3.3;                        % [-] - peakedness factor
omega_= linspace(0.001,2.0,N);      % [Hz] - frequency range

x = 0.0;                            % [m] - po sition with regard to wave propagation
t_max = 50.0;                       % [s] - maximum timeframe
t = linspace(0,t_max,(t_max*20));   % [s] - straigthened timeframe
z = linspace(-h,0,h*20);            % [m] - vertical cordiante (positive upwards)

Cd = 0.7;                           % [-] - drag coefficient
Cm = 2.0;                           % [-] - mass coefficient
D = 2.0;                            % [m] - diameter of monopile

%% Determining wave spectrum acc. to JONSWAP for limited fetch

T_p = 1.407*T_z;                    % [s] - mean peak period
omega_p = (2*pi)/T_p;               % [Hz] - circular frequency at spectral peak

for i=1:length(omega_);             % [-] - step function of omega
    
    if omega_(i)<omega_p;
        sigma(i)=0.07;
    else
        sigma(i)=0.09;
    end
    
end

A = exp(-(((omega_./omega_p)-1)./(sigma.*sqrt(2))).^2); % [-] - JONSWAP factor

S_J = ((320*H_s^2)/T_p^4).*omega_.^(-5).*...            % [m^2s] - wave spectrum
    exp((-1950/T_p^4).*omega_.^(-4)).*gamma.^A; 

fig1 = 'Linear wave theory - wave spectra';
figure('Name',fig1,'NumberTitle','off')

plot(omega_,S_J,'--','linewidth',2)
grid on
xlabel('Wave frequency, \omega [rad/s]')
ylabel('Spectral density, S(\omega) [m^2s]')
hold on

% Spectrum analysis and Hs check-up
m_J0 = trapz(omega_,S_J);      % [m^2] - area under spectral curve
RMS_J = sqrt(m_J0);            % [m] - root mean square of the water surface elevation
H_Jcon = 4*RMS_J;              % [m] - approx. significant wave height
zeta_a_Jcon = 2*RMS_J;         % [m] - approx. significant wave amplitude

%% Determining wave spectrum acc. to Bretschneider for open sea

T_1 = 0.772*T_p;                              % [s] - mean centroid wave period (modified two-paramter Pierson-Moskowitz)

S_B = ((173*H_s^2)/T_1^4).*omega_.^(-5).*...  % [m^2s] - wave spectrum
      exp((-692/T_1^4).*omega_.^(-4)); 

plot(omega_,S_B,'-r','linewidth',2)
legend('JONSWAP spectrum','Bretschneider spectrum')
text(0.82,1.5,['T_p = ' num2str(round(T_p)),'.0 sec']);
text(0.04,3.5,['H_s = ' num2str(H_s),'.0 m']);
hold off

% Spectrum analysis and Hs check-up
m_B0 = trapz(omega_,S_B);      % [m^2] - area under spectral curve
RMS_B = sqrt(m_B0);            % [m] - root mean square of the water surface elevation
H_Bcon = 4*RMS_B;              % [m] - approx. significant wave height
zeta_a_Bcon = 2*RMS_B;         % [m] - approx. significant wave amplitude

%% Generating regular wave components

delta_omega = omega_(2)-omega_(1);           % [Hz] - frequency interval

zeta_a_ = 2.*sqrt(S_J.*delta_omega);         % [m] - wave component amplitudes

% Solving dispersion relationship for the specified frequency band
for n=1:length(omega_)
    
    k0 = 0.1;
    k(:,n) = fsolve(@(k)Wavenumber(k,omega_(n),g,h),k0);       % [rad/m] - component wavenumbers
    
end

% Defining phase angle band
phi = deg2rad(randi([0 360],1,N));                             % [rad] - component phase shifts

% Calculating surface elevation
for j=1:length(omega_)
    
    zeta_(j,:) = zeta_a_(j).*sin(omega_(j).*t-k(j)*x+phi(j));  % [m] - wave component profiles
    
end

%% Realising irregular sea state

% Superimposing of regular wave components
wave_irr = sum(zeta_);                                         % [m] - irregular wave surface elevation

fig2 = 'Linear wave theory - irregular wave';
figure('Name',fig2,'NumberTitle','off')

plot(t,wave_irr,'linewidth',2)
xlabel('Time, t [sec]')
ylabel('Wave profile, \eta [m]')
grid on
graph2d.constantline(0);
text((t_max-1.8),0.15,'SWL','FontSize',6)

%% Plotting wave superposition of irregular wave

samples = randi([(N/2) N],1,numb_exp);           % [-] - example wave components

for q=1:length(samples)
    
    p = samples(q);    
    wave_exp(q,:) = zeta_(p,:);
    
end

% Plotting only four example components and final irregular wave elevation
fig3 = 'Linear wave theory - wave superposition';
figure('Name',fig3,'NumberTitle','off')

subplot(numb_exp+1,1,1)
title('Regular wave components')
hold on

for r=1:length(samples)
        
    subplot(numb_exp+1,1,r);    
    plot(t,wave_exp(r,:),'-c','linewidth',2)    
    ylabel('Wave profile, \eta [m]')
    graph2d.constantline(0);
    grid on
    hold on
    
end

subplot(numb_exp+1,1,numb_exp+1);
plot(t,wave_irr,'-b','linewidth',2)
graph2d.constantline(0);
ylabel('Wave profile, \eta [m]')
xlabel('Time, t [sec]')
title('Irregular wave')
grid on
hold off

%% Displaying Hs check-up and comparison

disp('Validation ->');
disp(['measured H_s: ',num2str(H_s),' m']);
disp(['approx. H_s: ',num2str(H_Jcon),' m (JONSWAP)']);
disp(['approx. H_s: ',num2str(H_Bcon),' m (Bretschneider)']);

%% Determining wave kinematics for regular wave components at SWL

for ii=1:length(omega_)
    
    [t_,z_]=meshgrid(t,z);
    
    pD_(:,:,ii) = rho.*g.*zeta_a_(ii).*((cosh(k(ii).*(z_+h)))/(cosh(k(ii).*h))).*sin(omega_(ii).*t_-k(ii).*x+phi(ii));         % [Pa] - dynamic pressure
    
    u_(:,:,ii) = omega_(ii).*zeta_a_(ii).*((cosh(k(ii).*(z_+h)))/(sinh(k(ii).*h))).*sin(omega_(ii).*t_-k(ii).*x+phi(ii));      % [m/s] - x-component velocity
    
    w_(:,:,ii) = omega_(ii).*zeta_a_(ii).*((sinh(k(ii).*(z_+h)))/(sinh(k(ii).*h))).*cos(omega_(ii).*t_-k(ii).*x+phi(ii));      % [m/s] - y-component velocity
    
    a1_(:,:,ii) = omega_(ii).^2.*zeta_a_(ii).*((cosh(k(ii).*(z_+h)))/(sinh(k(ii).*h))).*cos(omega_(ii).*t_-k(ii).*x+phi(ii));  % [m/^2] - x-component acceleration
    
    a3_(:,:,ii) = -omega_(ii).^2.*zeta_a_(ii).*((sinh(k(ii).*(z_+h)))/(sinh(k(ii).*h))).*sin(omega_(ii).*t_-k(ii).*x+phi(ii)); % [m/^2] - y-component acceleration
       
end

%% Generating irregular wave kinematics

% Superimposing regular wave properties
pD_irr=sum(pD_,3);              % [Pa] - dynamic pressure

u_irr=sum(u_,3);                % [m/s] - x-component velocity

w_irr=sum(w_,3);                % [m/s] - z-component velocity

a1_irr=sum(a1_,3);              % [m/s^2] - x-component acceleration

a3_irr=sum(a3_,3);              % [m/s^2] - z-component acceleration

fig4 = 'Linear wave theory - irregular superimposed x-component velocity';
figure('Name',fig4,'NumberTitle','off')

surf(t,z,u_irr,'EdgeColor','none')
cb = colorbar;
ylabel(cb,'x-component velocity, u [m/2]');
xlabel('Time, t [sec]')
ylabel('Water depth, d [m]')
zlabel('Velocity, u [m/2]')

MWL = length(z);                % [-] - position of mean waterlevel
ymin = -H_s-2;                  % [-] - minimum scale on y-axis
ymax = H_s+2;                   % [-] - maximum scale on y-axis

fig5 = 'Linear wave theory - irregular wave elevation and kinematics';
figure('Name',fig5,'NumberTitle','off')

subplot(3,2,1);
plot(t,wave_irr,'-b','linewidth',2)
ylabel('Wave profile, \eta [m]')
grid on
ticks = linspace(ymin,ymax,5);
set(gca,'YTickMode','manual','YTick',ticks,'ylim',[ymin,ymax]);
graph2d.constantline(0);

subplot(3,2,2);
plot(t,pD_irr(MWL,:),'-c','linewidth',2)
ylabel('Dynamic pressure, p_D [Pa]')
ticks2 = (linspace(ymin,ymax,5))*10^4;
set(gca,'YTickMode','manual','YTick',ticks2,'ylim',[ymin*10^4,ymax*10^4]);
grid on
graph2d.constantline(0);

subplot(3,2,3);
plot(t,u_irr(MWL,:),'-c','linewidth',2)
ylabel('x-velocity, u [m]')
set(gca,'YTickMode','manual','YTick',ticks,'ylim',[ymin,ymax]);
grid on
graph2d.constantline(0);

subplot(3,2,4);
plot(t,w_irr(MWL,:),'-c','linewidth',2)
ylabel('z-velocity, w [m]')
set(gca,'YTickMode','manual','YTick',ticks,'ylim',[ymin,ymax]);
grid on
graph2d.constantline(0);

subplot(3,2,5);
plot(t,a1_irr(MWL,:),'-c','linewidth',2)
xlabel('Time, t [sec]')
ylabel('x-acceleration, a_1 [m/s^2]')
set(gca,'YTickMode','manual','YTick',ticks,'ylim',[ymin,ymax]);
grid on
graph2d.constantline(0);

subplot(3,2,6);
plot(t,a3_irr(MWL,:),'-c','linewidth',2)
xlabel('Time, t [sec]')
ylabel('z-acceleration,a_3 [m/s^2]')
set(gca,'YTickMode','manual','YTick',ticks,'ylim',[ymin,ymax]);
grid on
graph2d.constantline(0);

%% Profile extension for wave kinematics - Wheeler stretching

% Calculating new grid coordinates in vertical direction
for jj=1:length(wave_irr);
   
    zs(:,jj) = ((z.*(h+wave_irr(jj)))/h)+wave_irr(jj);             % [m] - stretching of original z-coordinates
    
end

[t_new,z_new]=meshgrid(linspace(0,t_max,(t_max*20)),-h:0.1:H_s+2); % [s,m] - new meshgrid for interpolating stretched z-coordinates

% Interpolation of wave kinematics between original and new z-coordinates
for ii=1:length(t)
    
    pD_new(:,ii)=interp1(zs(:,ii),pD_irr(:,ii),z_new(:,ii));
    u_new(:,ii)=interp1(zs(:,ii),u_irr(:,ii),z_new(:,ii));
    w_new(:,ii)=interp1(zs(:,ii),w_irr(:,ii),z_new(:,ii));
    a1_new(:,ii)=interp1(zs(:,ii),a1_irr(:,ii),z_new(:,ii));
    a3_new(:,ii)=interp1(zs(:,ii),a3_irr(:,ii),z_new(:,ii));
    
end

fig6 = 'Linear wave theory - Wheeler stretching of x-component velocity';
figure('Name',fig6,'NumberTitle','off');

% Plotting example velocity for Wheeler stretching
exp = length(t)/2;                                               
zs_exp = zs(size(zs,1),exp);

plot(u_irr(:,exp),zs(:,exp),'-r',u_irr(:,exp),z,'-b','linewidth',2)
graph2d.constantline(zs_exp,'LineStyle','--','Color',[0 0 1]);
set(gca,'ylim',[-h, max(z_new(:,exp))]);
grid on
xlabel('x-component velocity, u [m/2]')
ylabel('Water depth, d [m]')
legend('Stretched kinematics (Wheeler)','Original kinematics (Linear theory)','Instantaneous surface elevation')
text((min(u_irr(:,exp)+0.005)),-9,['Time = ' num2str(round(t(1,exp))),'.0 sec']);
graph2d.constantline(0);
text((min(u_irr(:,exp)-0.005)),0.4,'SWL','FontSize',6)

%% Plotting contour plot for total water particle velocity

v_res = sqrt(u_new.^2+w_new.^2);                      % [m/s] - resultant velocity

fig7 = 'Linear wave theory - contour plot of water particle velocities';
figure('Name',fig7,'NumberTitle','off');

subplot(3,1,1)
contourf(t_new,z_new,u_new,50,'EdgeColor','none')
grid on
ylabel('Water depth, d [m]')
cb = colorbar;
ylabel(cb,'x-component velocity, u [m/s]');
graph2d.constantline(0);
text((max(t(1,:))-2),0.8,'SWL','FontSize',6)

subplot(3,1,2)
contourf(t_new,z_new,w_new,50,'EdgeColor','none')
grid on
ylabel('Water depth, d [m]')
cb = colorbar;
ylabel(cb,'z-component velocity, w [m/s]');
graph2d.constantline(0); 
text((max(t(1,:))-2),0.8,'SWL','FontSize',6)

subplot(3,1,3)
contourf(t_new,z_new,v_res,50,'EdgeColor','none')
grid on
xlabel('Time ,t [sec]')
ylabel('Water depth, d [m]')
cb = colorbar;
ylabel(cb,'Resultant velocity, v [m/s]');
graph2d.constantline(0);
text((max(t(1,:))-2),0.8,'SWL','FontSize',6)

%% Plotting contour plot for total water particle velocity

a_res = sqrt(a1_new.^2+a3_new.^2);                      % [m/s] - resultant velocity

fig8 = 'Linear wave theory - contour plot of water particle acceleration';
figure('Name',fig8,'NumberTitle','off');

subplot(3,1,1)
contourf(t_new,z_new,a1_new,50,'EdgeColor','none')
grid on
ylabel('Water depth, d [m]')
cb = colorbar;
ylabel(cb,'x-component acc., a_1 [m/s^2]');
graph2d.constantline(0);
text((max(t(1,:))-2),0.8,'SWL','FontSize',6)

subplot(3,1,2)
contourf(t_new,z_new,a3_new,50,'EdgeColor','none')
grid on
ylabel('Water depth, d [m]')
cb = colorbar;
ylabel(cb,'z-component acc., a_3 [m/s^2]');
graph2d.constantline(0); 
text((max(t(1,:))-2),0.8,'SWL','FontSize',6)

subplot(3,1,3)
contourf(t_new,z_new,a_res,50,'EdgeColor','none')
grid on
xlabel('Time ,t [sec]')
ylabel('Water depth, d [m]')
cb = colorbar;
ylabel(cb,'Resultant acc., a [m/s^2]');
graph2d.constantline(0);
text((max(t(1,:))-2),0.8,'SWL','FontSize',6)

%% Plotting quiver plot for velocity and acceleration field

% Data minimisation for quiver plot for visibility reasons
u_0 = u_new;                            
u_0(isnan(u_0))=0;
u_0(2:2:end,:)=0;
u_0(3:4:end,:)=0;
u_0(5:8:end,:)=0;
u_0(:,2:2:size(u_new,2))=0;
u_0(:,5:4:size(u_new,2))=0;

w_0 = w_new;                            
w_0(isnan(w_0))=0;
w_0(2:2:end,:)=0;
w_0(3:4:end,:)=0;
w_0(5:8:end,:)=0;
w_0(:,2:2:size(w_new,2))=0;
w_0(:,5:4:size(w_new,2))=0;

fig9 = 'Linear wave theory - velocity field for regular waves';
figure('Name',fig9,'NumberTitle','off');

I = ~(u_0==0 & w_0==0);
quiver(t_new(I),z_new(I),u_0(I),w_0(I),'Color',[1 0 0]);
axis([0 max(t) -h H_s])
grid on
hold on

plot(t(1,:),wave_irr,'-b','linewidth',2)
xlabel('Time, t [s]')
ylabel('Waterdepth, d [m]')
legend('Resultant velocity field','Surface elevation')
text((max(t(1,:))-2),0.2,'SWL','FontSize',6)
graph2d.constantline(0);
hold off

% Data minimisation for quiver plot for visibility reasons
a1_0 = a1_new;                        
a1_0(isnan(a1_0))=0;
a1_0(2:2:end,:)=0;
a1_0(3:4:end,:)=0;
a1_0(5:8:end,:)=0;
a1_0(:,2:2:size(a1_new,2))=0;
a1_0(:,5:4:size(a1_new,2))=0;

a3_0 = a3_new;                        
a3_0(isnan(a3_0))=0;
a3_0(2:2:end,:)=0;
a3_0(3:4:end,:)=0;
a3_0(5:8:end,:)=0;
a3_0(:,2:2:size(a3_new,2))=0;
a3_0(:,5:4:size(a3_new,2))=0;

fig10 = 'Linear wave theory - acceleration field for regular waves';
figure('Name',fig10,'NumberTitle','off');

I = ~(u_0==0 & w_0==0);
quiver(t_new(I),z_new(I),a1_0(I),a3_0(I),'Color',[0 1 0]);
axis([0 max(t) -h H_s])
grid on
hold on

plot(t(1,:),wave_irr,'-b','linewidth',2)
xlabel('Time, t [s]')
ylabel('Waterdepth, d [m]')
legend('Resultant acceleration field','Surface elevation')
text((max(t(1,:))-2),0.2,'SWL','FontSize',6)
graph2d.constantline(0);
hold off

%% Implementation of Morison's equation

% Calling external Morison's equation
[Fd,Fi,Fs] = MorisonLoad (Cd,Cm,rho,D,z_new,u_new,a1_new);

% Integration of wave load components along monopile length
F_drag=nansum(Fd);             % [N] - summed drag force along depth

F_inertia = nansum(Fi);        % [N] - summed inertia force along depth

F_shear = F_drag+F_inertia;    % [N] - summed shear/wave load along depth

% Calculating overturning moment with respect to mudline
z_h = z_new+h;                 % [m] - lever arm for OTM with respect to seabed

OTM_= Fs.*z_h;                 % [Nm] - overturning moment

OTM=nansum(OTM_);              % [Nm] - summed overturning moment along depth

% Identifying maximum structural response
[OTMmax,pos] = max(abs(OTM));       % [Nm] - maximum overturning moment

tmax = t(1,pos);               % [s] - point in time of maximum overturning moment

disp('Results ->');
disp(['Maximum overturning moment: ',num2str(OTMmax/1000,'%0.2f'),' KNm']);
disp(['Point in time of maximum overturning moment: ',num2str(tmax,'%0.1f'),' sec']);

fig11 = 'Wave loads over depth - Morison force and overturning moment';
figure('Name',fig11,'NumberTitle','off');

subplot(1,2,1)
plot(Fd(:,exp)/1000,z_new(:,1),'-y',Fi(:,exp)/1000,z_new(:,1),'-m',Fs(:,exp)/1000,z_new(:,1),'-r','LineWidth',2)
% axis([0 200 -h max(z_new(:,1))]);
% ticks3 = linspace(0,200,11);
% set(gca,'XTickMode','manual','XTick',ticks3,'xlim',[0,200]);
graph2d.constantline(zs_exp,'LineStyle','--','Color',[0 0 1]);
legend('Drag component','Inertia component','Total/shear force','Instantaneous surface elevation');
xlabel('Wave loads, F [kN]')
ylabel('Water depth, d [m]')
graph2d.constantline(0);
text(min(Fs(:,exp)/1000)+0.01,-h+1,['Time = ' num2str(round(t(1,exp))),'.0 sec']);
text(0.8-0.05,0.2,'SWL','FontSize',6);
grid on

subplot(1,2,2)
plot(OTM_(:,exp)/1000,z_new(:,1),'-g','LineWidth',2)
% axis([0 2000 -h max(z_new(:,1))]);
% ticks4 = linspace(0,2000,11);
% set(gca,'XTickMode','manual','XTick',ticks4,'xlim',[0,2000]);
graph2d.constantline(zs_exp,'LineStyle','--','Color',[0 0 1]);
legend('Overturning moment','Instantaneous surface elevation')
xlabel('Overturning moment, OTM [kNm]')
ylabel('Water depth, d [m]')
graph2d.constantline(0);
text(min(OTM_(:,exp)/1000)+0.1,-h+1,['Time = ' num2str(round(t(1,exp))),'.0 sec']);
text(8-0.5,0.2,'SWL','FontSize',6);
grid on

fig12 = 'Wave loads over time - Morison force and overturning moment';
figure('Name',fig12,'NumberTitle','off');

subplot(3,1,1);
plot(t(1,:),wave_irr,'-b','linewidth',2)
ylabel('Wave profile, \eta [m]')
grid on
ticks = linspace(ymin,ymax,5);
set(gca,'YTickMode','manual','YTick',ticks,'ylim',[ymin,ymax]);
text((min(t(1,:))+1),(min(wave_irr)-2.5),['d = ' num2str(h),'.0 m','    H_s = ' num2str(H_s),'.0 m','    T_z = ' num2str(T_z),'.0 m'])
graph2d.constantline(0);

subplot(3,1,2);
plot(t(1,:),F_drag/1000,'-y',t(1,:),F_inertia/1000,'-m',t(1,:),F_shear/1000,'-r','linewidth',2)
ylabel('Wave loads, F [kN]')
legend('Drag component (C_D = 0.7)','Inertia component (C_M = 2.0)','Total/shear force','Location','NorthWest')
grid on
graph2d.constantline(0);

subplot(3,1,3);
plot(t(1,:),OTM/1000,'-g','linewidth',2)
% axis([0 max(t(1,:)) -2*10^5 2*10^5]);
ylabel('Overturning moment, OTM [kNm]')
xlabel('Time, t [sec]')
grid on
graph2d.constantline(0);
text((min(t(1,:))+1),-750,['Diameter of monopile = ' num2str(D),'.0 m','          OTM_{max} = ' num2str(OTMmax/1000,'%0.1f'),' kNm']);

%% Data collection 

data(:,1)=wave_irr;
data(:,2)=t;
data(:,3)=OTM;

save('Irregular_Seastate.txt','data','-ASCII');

% End of code