%% Master Thesis - Non-linear Wave Impact on Monopile Structures

%  Philipp Schpfer - KTH (Stockholm) & NTNU (Trondheim)

%  In Corporation With Ramboll Denmark AS, Esbjerg

%  30.03.2016 -  LINEAR vs. NON-LINEAR WAVE THEORY
%  Regular wave - comparison between Linear and Stokes' second order
%  (different water depths)
 
clear all
close all
clc

%% Input data

g = 9.81;                                      % [m/s^2] - acceleration of gravity
rho = 1025.0;                                  % [kg/m^3] - salt water density

h = [10.0 20.0];                                      % [m] - average water depth
T = 10.0;                                      % [s] - wave period
H = 5.0;                                    % [m] - wave height

x = 0;                                         % [m] - position with regard to wave propagation
t = linspace(-T,T*2,100);                      % [s] - timeframe

Cd = 1.05;                                     % [-] - drag coefficient
Cm = 2.0;                                      % [-] - mass coefficient
D = 1.0;                                       % [m] - diameter of monopile

%% Pre-calculations: Solving dispersion relationship iteratively to determine wave number

zeta_a = H/2;                                  % [m] - wave amplitude
omega = (2*pi)/T;                              % [Hz] - wave frequency

for n=1:length(h)
    
    k0 = 0.1;
    k(:,n) = fsolve(@(k)Wavenumber(k,omega,g,h(n)),k0);       % [rad/m] - component wavenumbers
    
end

L = (2*pi)./k;

%% Determining Stokes' 1st order wave (linear wave theory)

for j=1:length(h)
    
    zeta_1(j,:) = zeta_a*cos(omega.*t-k(j).*x); % [m] - wave component profiles
    
end

%% Determining Stokes' 2nd order wave (non-linear wave theory

for i=1:length(h)
    
    zeta_2(i,:) = zeta_a*cos(omega.*t-k(i).*x)+(1/16).*k(i).*H^2*...
        (3*coth(k(i)*h(i))^3-coth(k(i)*h(i)))*cos(2*(omega.*t-k(i)*x)); % [m] - wave component profiles
    
end

%% Plotting linear and non-linear regular waves

fig1 = 'Non-linear wave theory - Stokes 1st and 2nd order wave';
figure('Name',fig1,'NumberTitle','off')

ymin = -H+1;                                         % [-] - minimum scale on y-axis
ymax = H-1;                                          % [-] - maximum scale on y-axis

subplot(4,1,1)
plot((t/T),zeta_1(1,:),'-b','linewidth',2)
ylabel('Wave profile, \eta [m]')
legend(['d = ' num2str(h(1)),'.0 m','    H = ' num2str(H),'.0 m','    T = ' num2str(T),'.0 m'])
grid on
ticks = linspace(ymin,ymax,5);
set(gca,'YTickMode','manual','YTick',ticks,'ylim',[ymin,ymax]);
title('Stokes` 1^s^t (linear theory)')
graph2d.constantline(0);

subplot(4,1,2)
plot((t/T),zeta_2(1,:),'-c','linewidth',2)
ylabel('Wave profile, \eta [m]')
legend(['d = ' num2str(h(1)),'.0 m','    H = ' num2str(H),'.0 m','    T = ' num2str(T),'.0 m'])
grid on
ticks = linspace(ymin,ymax,5);
set(gca,'YTickMode','manual','YTick',ticks,'ylim',[ymin,ymax]);
title('Stokes` 2^n^d')
graph2d.constantline(0);

subplot(4,1,3)
plot((t/T),zeta_1(2,:),'-b','linewidth',2)
ylabel('Wave profile, \eta [m]')
legend(['d = ' num2str(h(2)),'.0 m','    H = ' num2str(H),'.0 m','    T = ' num2str(T),'.0 m'])
grid on
ticks = linspace(ymin,ymax,5);
set(gca,'YTickMode','manual','YTick',ticks,'ylim',[ymin,ymax]);
title('Stokes` 1^s^t (linear theory)')
graph2d.constantline(0);

subplot(4,1,4)
plot((t/T),zeta_2(2,:),'-c','linewidth',2)
ylabel('Wave profile, \eta [m]')
xlabel('t/T [-]')
legend(['d = ' num2str(h(2)),'.0 m','    H = ' num2str(H),'.0 m','    T = ' num2str(T),'.0 m'])
title('Stokes` 2^n^d')
graph2d.constantline(0);
grid on
ticks = linspace(ymin,ymax,5);
set(gca,'YTickMode','manual','YTick',ticks,'ylim',[ymin,ymax]);

 % End of code