%% Master Thesis - Non-linear Wave Impact on Monopile Structures

%  Philipp Schpfer - KTH (Stockholm) & NTNU (Trondheim)

%  In Corporation With Ramboll Denmark AS, Esbjerg

%  30.03.2016 -  LINEAR vs. NON-LINEAR WAVE THEORY
%  Regular wave - comparison between Linear and Stokes' second order
%  (different wave heights)
 
clear all
close all
clc

%% Input data

g = 9.81;                                      % [m/s^2] - acceleration of gravity
rho = 1025.0;                                  % [kg/m^3] - salt water density

h = 10.0;                                      % [m] - average water depth
T = 10.0;                                      % [s] - wave period
H = [5 15];                                    % [m] - wave height

x = 0;                                         % [m] - position with regard to wave propagation
t = linspace(-T,T*2,100);                      % [s] - timeframe

Cd = 1.05;                                     % [-] - drag coefficient
Cm = 2.0;                                      % [-] - mass coefficient
D = 1.0;                                       % [m] - diameter of monopile

%% Pre-calculations: Solving dispersion relationship iteratively to determine wave number

zeta_a = H/2;                                  % [m] - wave amplitude
omega = (2*pi)/T;                              % [Hz] - wave frequency

k0 = 0.1;                                      % [-] - initial value for wave number solver 
k = fsolve(@(k)Wavenumber(k,omega,g,h),k0);    % [rad/m] - wavenumber
L = (2*pi)/k;

%% Determining Stokes' 1st order wave (linear wave theory)

for j=1:length(H)
    
    zeta_1(j,:) = zeta_a(j)*cos(omega.*t-k*x); % [m] - wave component profiles
    
end

%% Determining Stokes' 2nd order wave (non-linear wave theory

for i=1:length(H)
    
    zeta_2(i,:) = zeta_a(i)*cos(omega.*t-k*x)+(1/16)*k*H(i)^2*...
        (3*coth(k*h)^3-coth(k*h))*cos(2*(omega.*t-k*x)); % [m] - wave component profiles
    
end

%% Plotting linear and non-linear regular waves

fig1 = 'Non-linear wave theory - Stokes 1st and 2nd order wave';
figure('Name',fig1,'NumberTitle','off')

ymin = -H(2)+1;                                         % [-] - minimum scale on y-axis
ymax = H(2)-1;                                          % [-] - maximum scale on y-axis

subplot(4,1,1)
plot((t/T),zeta_1(1,:),'-b','linewidth',2)
ylabel('Wave profile, \zeta [m]')
legend(['h = ' num2str(h),'.0 m','    H = ' num2str(H(1)),'.0 m','    T = ' num2str(T),'.0 m'])
grid on
ticks = linspace(ymin,ymax,5);
set(gca,'YTickMode','manual','YTick',ticks,'ylim',[ymin,ymax]);
title('Stokes 1^s^t (linear theory)')
graph2d.constantline(0);

subplot(4,1,2)
plot((t/T),zeta_2(1,:),'-c','linewidth',2)
ylabel('Wave profile, \zeta [m]')
legend(['h = ' num2str(h),'.0 m','    H = ' num2str(H(1)),'.0 m','    T = ' num2str(T),'.0 m'])
grid on
ticks = linspace(ymin,ymax,5);
set(gca,'YTickMode','manual','YTick',ticks,'ylim',[ymin,ymax]);
title('Stokes 2^n^d')
graph2d.constantline(0);

subplot(4,1,3)
plot((t/T),zeta_1(2,:),'-b','linewidth',2)
ylabel('Wave profile, \zeta [m]')
legend(['h = ' num2str(h),'.0 m','    H = ' num2str(H(2)),'.0 m','    T = ' num2str(T),'.0 m'])
grid on
ticks = linspace(ymin,ymax,5);
set(gca,'YTickMode','manual','YTick',ticks,'ylim',[ymin,ymax]);
title('Stokes 1^s^t (linear theory)')
graph2d.constantline(0);

subplot(4,1,4)
plot((t/T),zeta_2(2,:),'-c','linewidth',2)
ylabel('Wave profile, \zeta [m]')
xlabel('t/T [-]')
legend(['h = ' num2str(h),'.0 m','    H = ' num2str(H(2)),'.0 m','    T = ' num2str(T),'.0 m'])
title('Stokes 2^n^d')
graph2d.constantline(0);
grid on
ticks = linspace(ymin,ymax,5);
set(gca,'YTickMode','manual','YTick',ticks,'ylim',[ymin,ymax]);

 % End of code