%% Master Thesis - Non-linear Wave Impact on Monopile Structures

%  Philipp Schpfer - KTH (Stockholm) & NTNU (Trondheim)

%  In Corporation With Ramboll Denmark AS, Esbjerg

%  04.02.2016 -  NON-LINEAR WAVE THEORY
%  Regular wave - Stokes' second order
 
clear all
close all
clc

%% Input data

g = 9.81;               % [m/s^2] - acceleration of gravity
rho = 1025.0;           % [kg/m^3] - salt water density
x = 0.0;                % [m] - position with regard to wave propagation
h = 20.0;               % [m] - average water depth

T = 10.0;               % [s] - wave period
H = 5.0;                % [m] - wave height

Cd = 0.7;               % [-] - drag coefficient
Cm = 2.0;               % [-] - mass coefficient
D = 2.0;                % [m] - diameter of monopile

%% Pre-calculations: Solving dispersion relationship iteratively to determine wave number

zeta_a = H/2;                                            % [m] - wave amplitude
omega = (2*pi)/T;                                        % [Hz] - wave frequency

k0 = 0.1;
k = fsolve(@(k)Wavenumber(k,omega,g,h),k0);              % [rad/m] - wavenumber
lambda = (2*pi)/k;

[t,z]=meshgrid(linspace(0,T*2,100),linspace(-h,0,50));   % [s,m] - time and vertical coordinate

%% Determining wave profile and kinematics at SWL

zeta = (H/2).*cos(k*x-omega.*t)...
    +((pi*H^2)/(8*lambda)).*(cosh(k*h)/sinh(k*h)^3).*(2+cosh(2*k*h)).*cos(2.*(k*x-omega.*t));                                           % [m] - wave profile

pD = 0.5*rho*g*H.*((cosh(k.*(z+h)))/(cosh(k*h))).*cos(k*x-omega.*t)...
    +0.75*rho*g*H.*((pi*H)/(lambda*sinh(2*k*h))).*((cosh(2*k.*(z+h)))/(sinh(k*h)^2)-1/3).*cos(2.*(k*x-omega.*t))...
    -0.25*rho*g*H.*((pi*H)/(lambda*sinh(2*k*h))).*((cosh(2*k.*(z+h)))-1);   % [Pa] - dynamic pressure

u = ((pi*H)/T).*((cosh(k.*(z+h)))/(sinh(k*h))).*cos(k*x-omega.*t)...
    +0.75*((pi*H)/T)*((pi*H)/lambda).*((cosh(2*k.*(z+h)))/(sinh(k*h)^4)).*cos(2.*(k*x-omega.*t));     % [m/s] - x-component of velocity

w = ((pi*H)/T).*((sinh(k.*(z+h)))/(sinh(k*h))).*sin(k*x-omega.*t)...
    +0.75*((pi*H)/T)*((pi*H)/lambda).*((sinh(2*k.*(z+h)))/(sinh(k*h)^4)).*sin(2.*(k*x-omega.*t));     % [m/s] - x-component of velocity

a1 = ((2*pi^2*H)/T^2).*((cosh(k.*(z+h)))/(sinh(k*h))).*sin(k*x-omega.*t)...
    +((3*pi^2*H)/T^2).*((pi*H)/lambda).*((cosh(2*k.*(z+h)))/(sinh(k*h)^4)).*sin(2.*(k*x-omega.*t));  % [m/s^2] - x-component of acceleration

a3 = -((2*pi^2*H)/T^2).*((sinh(k.*(z+h)))/(sinh(k*h))).*cos(k*x-omega.*t)...
    -((3*pi^2*H)/T^2).*((pi*H)/lambda).*((sinh(2*k.*(z+h)))/(sinh(k*h)^4)).*cos(2.*(k*x-omega.*t));  % [m/s^2] - x-component of acceleration

fig1 = 'Non-linear wave theory - wave profile and kinematics at mean waterlevel';
figure('Name',fig1,'NumberTitle','off');

ymin = -H;                                           % [-] - minimum scale on y-axis
ymax = H;                                            % [-] - maximum scale on y-axis
MWL = size(z,1);                                     % [-] - position of mean water level

zeta_MWL = zeta(MWL,:);
save('Second_shallow.mat','zeta_MWL')

subplot(3,2,1);
plot(t(1,:),zeta(MWL,:),'-b','linewidth',2)
ylabel('Wave profile, \eta [m]')
grid on
ticks = linspace(ymin,ymax,5);
set(gca,'YTickMode','manual','YTick',ticks,'ylim',[ymin,ymax]);
graph2d.constantline(0);

subplot(3,2,2);
plot(t(1,:),pD(MWL,:),'-b','linewidth',2)
ylabel('Dynamic pressure, p_D [Pa]')
ticks2 = (linspace(ymin,ymax,5))*10^4;
set(gca,'YTickMode','manual','YTick',ticks2,'ylim',[ymin*10^4,ymax*10^4]);
grid on
graph2d.constantline(0);

subplot(3,2,3);
plot(t(1,:),u(MWL,:),'-b','linewidth',2)
ylabel('x-velocity, u [m]')
set(gca,'YTickMode','manual','YTick',ticks,'ylim',[ymin,ymax]);
grid on
graph2d.constantline(0);

subplot(3,2,4);
plot(t(1,:),w(MWL,:),'-b','linewidth',2)
ylabel('z-velocity, w [m]')
set(gca,'YTickMode','manual','YTick',ticks,'ylim',[ymin,ymax]);
grid on
graph2d.constantline(0);

subplot(3,2,5);
plot(t(1,:),a1(MWL,:),'-b','linewidth',2)
xlabel('Time, t [sec]')
ylabel('x-acceleration, a_1 [m/s^2]')
set(gca,'YTickMode','manual','YTick',ticks,'ylim',[ymin,ymax]);
grid on
graph2d.constantline(0);

subplot(3,2,6);
plot(t(1,:),a3(MWL,:),'-b','linewidth',2)
xlabel('Time, t [sec]')
ylabel('z-acceleration,a_3 [m/s^2]')
set(gca,'YTickMode','manual','YTick',ticks,'ylim',[ymin,ymax]);
grid on
graph2d.constantline(0);

%% Profile extension for wave kinematics - Wheeler stretching

% Calculating new grid coordinates in vertical direction
for jj=1:length(zeta);
   
    zs(:,jj) = ((z(:,1)*(h+zeta(1,jj)))/h)+zeta(1,jj); % [m] - stretching of original z-coordinates
    
end

[t_new,z_new]=meshgrid(linspace(0,T*2,100),-h:0.1:H+2); % [s,m] - new meshgrid for interpolating stretched z-coordinates

% Interpolation of wave kinematics between original and new z-coordinates
for ii=1:length(t)
    
    pD_new(:,ii)=interp1(zs(:,ii),pD(:,ii),z_new(:,ii));
    u_new(:,ii)=interp1(zs(:,ii),u(:,ii),z_new(:,ii));
    w_new(:,ii)=interp1(zs(:,ii),w(:,ii),z_new(:,ii));
    a1_new(:,ii)=interp1(zs(:,ii),a1(:,ii),z_new(:,ii));
    a3_new(:,ii)=interp1(zs(:,ii),a3(:,ii),z_new(:,ii));
    
end

fig2 = 'Non-linear wave theory - Wheeler stretching of x-component velocity';
figure('Name',fig2,'NumberTitle','off');

% Plotting example velocity for Wheeler stretching
exp = length(t)/2;                                                 
zs_exp = zs(size(zs,1),exp);

plot(u(:,exp),zs(:,exp),'-r',u(:,exp),z(:,exp),'-b','linewidth',2)
ylim([-h max(z_new(:,exp))]);
graph2d.constantline(zs_exp,'LineStyle','--','Color',[0 0 1]);
grid on
xlabel('x-component velocity, u [m/2]')
ylabel('Water depth, h [m]')
legend('Stretched kinematics (Wheeler)','Original kinematics (Linear theory)','Instantaneous surface elevation')
text(min(u(:,exp))+0.05,-h+1,['Time = ' num2str(round(t(1,exp))),'.0 sec']);
graph2d.constantline(0);
text((max(u(:,exp)+0.033)),0.19,'MWL','FontSize',6)

%% Plotting contour plot for total water particle velocity

v_res = sqrt(u_new.^2+w_new.^2);                      % [m/s] - resultant velocity

fig3 = 'Non-linear wave theory - contour plot of water particle velocities';
figure('Name',fig3,'NumberTitle','off');

subplot(3,1,1)
contourf(t_new,z_new,u_new,50,'EdgeColor','none')
grid on
ylabel('Water depth, h [m]')
cb = colorbar;
ylabel(cb,'x-component velocity, u [m/2]');
graph2d.constantline(0);
text((max(t(1,:))-0.25),0.8,'MWL','FontSize',6)

subplot(3,1,2)
contourf(t_new,z_new,w_new,50,'EdgeColor','none')
grid on
ylabel('Water depth, h [m]')
cb = colorbar;
ylabel(cb,'z-component velocity, w [m/2]');
graph2d.constantline(0); 
text((max(t(1,:))-0.25),0.8,'MWL','FontSize',6)

subplot(3,1,3)
contourf(t_new,z_new,v_res,50,'EdgeColor','none')
grid on
xlabel('Time ,t [sec]')
ylabel('Water depth, h [m]')
cb = colorbar;
ylabel(cb,'Resultant velocity, v [m/s]');
graph2d.constantline(0);
text((max(t(1,:))-0.25),0.8,'MWL','FontSize',6)

%% Plotting contour plot for total water particle acceleration

a_res = sqrt(a1_new.^2+a3_new.^2);                      % [m/s] - resultant acceleration

fig4 = 'Non-linear wave theory - contour plot of water particle accelerations';
figure('Name',fig4,'NumberTitle','off');

subplot(3,1,1)
contourf(t_new,z_new,a1_new,50,'EdgeColor','none')
grid on
ylabel('Water depth, h [m]')
cb = colorbar;
ylabel(cb,'x-component acceleration, a_1 [m/s^2]');
graph2d.constantline(0);
text((max(t(1,:))-0.25),0.8,'MWL','FontSize',6)

subplot(3,1,2)
contourf(t_new,z_new,a3_new,50,'EdgeColor','none')
grid on
ylabel('Water depth, h [m]')
cb = colorbar;
ylabel(cb,'z-component acceleration, a_3 [m/s^2]');
graph2d.constantline(0); 
text((max(t(1,:))-0.25),0.8,'MWL','FontSize',6)

subplot(3,1,3)
contourf(t_new,z_new,a_res,50,'EdgeColor','none')
grid on
xlabel('Time ,t [sec]')
ylabel('Water depth, h [m]')
cb = colorbar;
ylabel(cb,'Resultant acceleration, v [m/s^2]');
graph2d.constantline(0);
text((max(t(1,:))-0.25),0.8,'MWL','FontSize',6)

%% Plotting quiver plot for velocity and acceleration field

% Data minimisation for quiver plot for visibility reasons
u_0 = u_new;                                           
u_0(isnan(u_0))=0;
u_0(2:2:end,:)=0;
u_0(3:4:end,:)=0;
u_0(5:8:end,:)=0;
u_0(:,2:2:size(u_new,2))=0;
u_0(:,5:4:size(u_new,2))=0;

w_0 = w_new;                                           
w_0(isnan(w_0))=0;
w_0(2:2:end,:)=0;
w_0(3:4:end,:)=0;
w_0(5:8:end,:)=0;
w_0(:,2:2:size(w_new,2))=0;
w_0(:,5:4:size(w_new,2))=0;

a1_0 = a1_new;                                         
a1_0(isnan(a1_0))=0;
a1_0(2:2:end,:)=0;
a1_0(3:4:end,:)=0;
a1_0(5:8:end,:)=0;
a1_0(:,2:2:size(a1_new,2))=0;
a1_0(:,5:4:size(a1_new,2))=0;

a3_0 = a3_new;                                         
a3_0(isnan(a3_0))=0;
a3_0(2:2:end,:)=0;
a3_0(3:4:end,:)=0;
a3_0(5:8:end,:)=0;
a3_0(:,2:2:size(a3_new,2))=0;
a3_0(:,5:4:size(a3_new,2))=0;

fig5 = 'Non-linear wave theory - velocity and acceleration field for regular waves';
figure('Name',fig5,'NumberTitle','off');

plot(t(1,:),zeta(MWL,:),'-b','linewidth',2)
grid on
hold on

I = ~(u_0==0 & w_0==0);
quiver(t_new(I),z_new(I),u_0(I),w_0(I),'Color',[1 0 0]);
axis([0 T*2 -h (H-1)])
quiver(t_new(I),z_new(I),a1_0(I),a3_0(I),'Color',[0 1 0]);

xlabel('Time, t [s]')
ylabel('Waterdepth, h [m]')
legend('Surface elevation','Resultant velocity','Resultant acceleration')
text((max(t(1,:))-0.5),0.2,'MWL','FontSize',6)
graph2d.constantline(0);
hold off

%% Implementation of Morison's equation

% Calling external Morison's equation
[Fd,Fi,Fs] = MorisonLoad (Cd,Cm,rho,D,z_new,u_new,a1_new);

% Integration of wave load components along monopile length
F_drag=nansum(Fd);             % [N] - summed drag force along depth

F_inertia = nansum(Fi);        % [N] - summed inertia force along depth

F_shear = F_drag+F_inertia;    % [N] - summed shear/wave load along depth

% Calculating overturning moment with respect to mudline
z_h = z_new+h;                 % [m] - lever arm for OTM with respect to seabed

OTM_= Fs.*z_h;                 % [Nm] - overturning moment

OTM=nansum(OTM_);              % [Nm] - summed overturning moment along depth

% Identifying maximum structural response
[OTMmax,pos] = max(abs(OTM));       % [Nm] - maximum overturning moment

tmax = t(1,pos);               % [s] - point in time of maximum overturning moment

disp('Results ->');
disp(['Maximum overturning moment: ',num2str(OTMmax/1000,'%0.2f'),' KNm']);
disp(['Point in time of maximum overturning moment: ',num2str(tmax,'%0.1f'),' sec']);

fig6 = 'Wave loads over depth - Morison force and overturning moment';
figure('Name',fig6,'NumberTitle','off');

[Fdmax,pos] = max(Fd(:,exp));
zeta_exp = z_new(pos,1);
    
subplot(1,2,1)
plot(Fd(:,exp)/1000,z_new(:,1),'-y',Fi(:,exp)/1000,z_new(:,1),'-m',Fs(:,exp)/1000,z_new(:,1),'-r','LineWidth',2)
axis([0 0.5 -h max(z_new(:,1))]);
ticks3 = linspace(0,0.5,11);
set(gca,'XTickMode','manual','XTick',ticks3,'xlim',[0,0.5]);
graph2d.constantline(zeta_exp,'LineStyle','--','Color',[0 0 1]);
legend('Drag component','Inertia component','Total/shear force','Instantaneous surface elevation');
xlabel('Wave loads, F [kN]')
ylabel('Water depth, d [m]')
graph2d.constantline(0);
text(min(Fs(:,exp))/1000+0.01,-9,['Time = ' num2str(round(t(1,exp))),'.0 sec']);
text(0.5-0.022,0.2,'SWL','FontSize',6);
grid on

subplot(1,2,2)
plot(OTM_(:,exp)/1000,z_new(:,1),'-g','LineWidth',2)
axis([0 12 -h max(z_new(:,1))]);
ticks4 = linspace(0,12,11);
set(gca,'XTickMode','manual','XTick',ticks4,'xlim',[0,12]);
graph2d.constantline(zeta_exp,'LineStyle','--','Color',[0 0 1]);
legend('Overturning moment','Instantaneous surface elevation')
xlabel('Overturning moment, OTM [kNm]')
ylabel('Water depth, d [m]')
graph2d.constantline(0);
text(min(OTM_(:,exp))/1000+0.5,-9,['Time = ' num2str(round(t(1,exp))),'.0 sec']);
text(12-0.22,0.2,'SWL','FontSize',6);
grid on

fig7 = 'Wave loads over time - Morison force and overturning moment';
figure('Name',fig7,'NumberTitle','off');

subplot(3,1,1);
plot(t(1,:),zeta(MWL,:),'-b','linewidth',2)
ylabel('Wave profile, \eta [m]')
grid on
ticks = linspace(ymin,ymax,5);
set(gca,'YTickMode','manual','YTick',ticks,'ylim',[ymin,ymax]);
graph2d.constantline(0);
text((min(t(1,:))+.5),(min(zeta(MWL,:)-1.5)),['d = ' num2str(h),'.0 m','    H = ' num2str(H(1)),'.0 m','    T = ' num2str(T),'.0 m'])

subplot(3,1,2);
plot(t(1,:),F_drag/1000,'-y',t(1,:),F_inertia/1000,'-m',t(1,:),F_shear/1000,'-r','linewidth',2)
ylabel('Wave loads, F [kN]')
legend('Drag component (C_D = 0.7)','Inertia component (C_M = 2.0)','Total/shear force')
grid on
graph2d.constantline(0);

subplot(3,1,3);
plot(t(1,:),OTM/1000,'-g','linewidth',2)
ylabel('Overturning moment, OTM [kNm]')
xlabel('Time, t [sec]')
grid on
graph2d.constantline(0);
text((min(t(1,:))+0.5),-750,['Diameter of monopile = ' num2str(D),'.0 m','          OTM_{max} = ' num2str(OTMmax/1000,'%0.1f'),' kNm at ' num2str(tmax,'%0.1f'),' sec']);

data(:,1)=t(1,:);
data(:,2)=OTM;

save('Stokes2nd_OTM.txt','data','-ASCII');

% End of code