%% Master Thesis - Non-linear Wave Impact on Monopile Structures

%  Philipp Schpfer - KTH (Stockholm) & NTNU (Trondheim)

%  In Corporation With Ramboll Denmark AS, Esbjerg

%  18.04.2016 -  NON-LINEAR WAVE THEORY
%  Regular wave - Stokes' fifth order - space dependency

clear all
close all
clc

%% Input data

Hs = 9.0;               % [m] - significant wave height
Tp = 13;                % [s] - wave peak period

h = 10.0;               % [m] - water depth
H = 5.0;                % [m] - wave height
T = 10.0;               % [s] - wave period
g = 9.81;               % [m/s^2] - acceleration of gravity
rho = 1025.0;           % [kg/m^3] - salt water density

Dia = 5.0;              % [m] - diameter of monopile
Cm = 2.0;               % [-] - inertia coefficient
Cd = 0.7;               % [-] - drag coefficient

t = 0.0;                % [s] - point in time    
x = [-100:100];         % [x] - spatial frame

%% Pre-calculations: Solving dispersion relationship iteratively to determine wave number

omega = (2*pi)/T;       % [Hz] - wave frequency

k0 = 0.1;
k = fsolve(@(k)Wavenumber(k,omega,g,h),k0);           % [rad/m] - wavenumber
L0 = (2*pi)/k;

%% Determining of Stokes' 5th order coefficients

lambda = 0;

[A, B, C, D, E]=Stokes_Fifth_Coefficients(k,h,lambda);

%% Realising surface elevation

i = 1;
error(i) = 9999;
kn = zeros(1,20);

% Applying series expansion
while error(i)>=0.00001
    
    syms k1
    eqn = omega^2-g*k1*tanh(k1*h)*(1+lambda^2*C(1,1)+lambda^4*C(1,2)) == 0;
    k1 = solve(eqn,k1);
    k1 = double(k1);
    k = abs(k1);
    kn(i+1) = abs(k1);
    
    syms lambda1
    eqn = ((abs(k1)*H)/2)-(lambda1+lambda1^3*B(3,3)+lambda1^5*(B(3,5)+B(5,5))) == 0;
    lambda1 = abs(solve(eqn,lambda1));
    lambda1 = double( lambda1 );
    lambda = min(lambda1(real(lambda1)&imag(lambda1)==0));
    [A, B, C, D, E]=Stokes_Fifth_Coefficients(k,h,lambda);
    
    error(i+1)=abs((((kn(i+1)-kn(i))/(kn(i+1)))*100));
    i=i+1;
    
end

% Solving Stokes' fifth order coefficients
L = ((g*T^2)/(2*pi))*tanh(k*h)*(1+lambda^2*C(1,1)+lambda^4*C(1,2));
theta = (k.*x-omega.*t);
n = zeros(1,length(t));

% Adding all surface elevation of different order numbers
for i=1:length(E)
    eta = (1/k)*E(i)*cos(i*theta);
    ea = eta+n;
    n = ea;
end

fig1 = 'Non-linear wave theory - wave profile';
figure('Name',fig1,'NumberTitle','off');

plot(x,ea,'b-','LineWidth',2);
title('Stokes 5^t^h order')
legend(['d = ' num2str(h),'.0 m','    H = ' num2str(H),'.0 m','    T = ' num2str(T),'.0 m'])
grid on
xlabel('x-position, x [m]');
ylabel('Wave profile, \eta [m]') ;
graph2d.constantline(0);
text((max(t(1,:))-0.8),0.1,'MWL','FontSize',6)

eta_fifth = ea';
save('Matlab_Fifth.txt', 'eta_fifth', '-ASCII','-append');


%% Calculating wave kinematics and Morison's equation as well as overturning moment

c = L/T;
y = 50;
u_new = zeros(y,length(x));
a1_new = zeros(y,length(x));
w_new = zeros(y,length(x));
a3_new = zeros(y,length(x));

% Applying series expansion 
for m=1:length(x)
    
    z = linspace(ea(m),-h,y);
    u1 = zeros(1,length(z));
    u = zeros(1,length(z));
    a1 = zeros(1,length(z));
    acc = zeros(1,length(z));
    nn = zeros(1,length(z));
    nnn = zeros(1,length(z));
    w1 = zeros(1,length(z));
    w = zeros(1,length(z));
    a3 = zeros(1,length(z));
    acc3 = zeros(1,length(z));
    rr = zeros(1,length(z));
    rrr = zeros(1,length(z));
    
    for o=1:length(z)
        for j=1:length(D)
            
            u1(o) = c*j*D(j)*cosh(j*k*(z(o)+h))*cos(j*k*x(m));
            u(o) = u1(o)+nn(o);
            nn(o) = u(o);
            a1(o) = c*omega*j^2*D(j)*cosh(j*k*(z(o)+h))*sin(j*k*x(m));
            acc(o) = a1(o)+nnn(o);
            nnn(o) = acc(o);
            
            w1(o) = c*j*D(j)*sinh(j*k*(z(o)+h))*sin(j*k*x(m));
            w(o) = w1(o)+rr(o);
            rr(o) = w(o);
            a3(o) = -c*omega*j^2*D(j)*sinh(j*k*(z(o)+h))*cos(j*k*x(m));
            acc3(o) = a3(o)+rrr(o);
            rrr(o) = acc3(o);
            
        end
    end
    
    u_new(:,m) = u;
    a1_new(:,m) = acc;
    w_new(:,m) = w;
    a3_new(:,m) = acc3;
    
    [Fd_,Fi_,Fs_] = MorisonLoad (Cd,Cm,rho,Dia,z,u_new,a1_new);
    
    Fd(:,m) = Fd_(:,m);
    Fi(:,m) = Fi_(:,m);
    Fs(:,m) = Fs_(:,m);
    
    z_h = (z+h)';                        % [m] - lever arm for OTM with respect to seabed
    
    OTM_= Fs_(:,m).*z_h;                 % [Nm] - overturning moment
    
    OTM(:,m) = OTM_;
    
end

fig2 = 'Non-linear wave theory - wave kinematics at MWL';
figure('Name',fig2,'NumberTitle','off');

ymin = -H;                                         % [-] - minimum scale on y-axis
ymax = H;                                          % [-] - maximum scale on y-axis
MWL = 50;                                    % [-] - position of mean water level

subplot(2,1,1);
plot(x,u_new(MWL,:),'-b','linewidth',2)
ylabel('x-velocity, u [m/s]')
grid on
ticks = linspace(ymin,ymax,5);
set(gca,'YTickMode','manual','YTick',ticks,'ylim',[ymin,ymax]);
graph2d.constantline(0);

subplot(2,1,2);
plot(x,a1_new(MWL,:),'-b','linewidth',2)
xlabel('x-position, x [m]')
ylabel('x-acceleration, u [m/^2]')
set(gca,'YTickMode','manual','YTick',ticks,'ylim',[ymin,ymax]);
grid on
graph2d.constantline(0);

%% Post-processing of above series expansion of Morison's equation

% Integrationg of wave loads and OTM along monopile foundation
F_drag=sum(Fd);             % [N] - summed drag force along depth

F_inertia = sum(Fi);        % [N] - summed inertia force along depth

F_shear = F_drag+F_inertia;    % [N] - summed shear/wave load along depth

OTM_tot=sum(OTM);              % [Nm] - summed overturning moment along depth

% Identifying maximum structural response
[OTMmax,pos] = max(abs(OTM_tot));       % [Nm] - maximum overturning moment

xmax = x(1,pos);               % [s] - point in time of maximum overturning moment

disp('Results ->');
disp(['Maximum overturning moment: ',num2str(OTMmax/1000,'%0.2f'),' KNm']);
disp(['Point in space of maximum overturning moment: ',num2str(xmax,'%0.1f'),' m']);

fig3 = 'Wave loads over space - Morison force and overturning moment';
figure('Name',fig3,'NumberTitle','off');

subplot(3,1,1);
plot(x,ea,'-b','linewidth',2)
ylabel('Wave profile, \eta [m]')
grid on
ticks = linspace(ymin,ymax,5);
set(gca,'YTickMode','manual','YTick',ticks,'ylim',[ymin,ymax]);
graph2d.constantline(0);
text((min(x(1,:))+.5),(min(ea-0.8)),['d = ' num2str(h),'.0 m','    H = ' num2str(H(1)),'.0 m','    T = ' num2str(T),'.0 m'])

subplot(3,1,2);
plot(x,F_drag/1000,'-y',x,F_inertia/1000,'-m',x,F_shear/1000,'-r','linewidth',2)
ylabel('Wave loads, F [KN]')
legend('Drag component','Inertia component','Total/shear force')
grid on
graph2d.constantline(0);

subplot(3,1,3);
plot(x,OTM_tot/1000,'-g','linewidth',2)
ylabel('Overturning moment, OTM [KNm]')
xlabel('x-position, x [m]')
grid on
graph2d.constantline(0);
text((min(x)+0.5),(min(OTM_tot)/1000-10),['Diameter of monopile = ' num2str(Dia),'.0 m','          OTM_{max} = ' num2str(OTMmax/1000,'%0.1f'),' KNm']);

% End of code